from django.conf import settings
from django.core.mail import send_mail

from singular_server_management.management.commands.server_base_command import ServerBaseCommand

PROJECT_NAME = getattr(settings, 'PROJECT_NAME', '')
HOST = getattr(settings, 'HOST', '')
USER = getattr(settings, 'USER', '')
GIT_PASSWORD = getattr(settings, 'GIT_PASSWORD', '')
PASSWORD = getattr(settings, 'PASSWORD', '')
KEY_FILE = getattr(settings, 'KEY_FILE', '')

if PASSWORD:
    UPDATE_CRONJOB = ('1 0 * * *', 'django.core.management.call_command',
                      [
                          f'update_server {PROJECT_NAME} {HOST} {USER} {GIT_PASSWORD} --server_password {PASSWORD}']),
else:
    UPDATE_CRONJOB = ('1 0 * * *', 'django.core.management.call_command',
                      [
                          f'update_server {PROJECT_NAME} {HOST} {USER} {GIT_PASSWORD} --key_file {KEY_FILE}']),


class Command(ServerBaseCommand):
    help = 'Update test server'

    def handle(self, *args, **options):
        super(Command, self).handle(*args, **options)
        try:
            with self.connection.cd(f'/var/www/django_apps/{self.project_name}/{self.project_name}'):
                self.update_project()
            self.restart_services()
        except Exception as e:
            send_mail(f'Failed to update {self.host} on project {self.project_name}', str(e),
                      settings.DEFAULT_FROM_EMAIL,
                      settings.ADMINS)



from django.core.management import BaseCommand
from fabric import Connection
from invoke import Responder


class ServerBaseCommand(BaseCommand):
    str_params = ['project_name', 'host', 'user', 'git_user_password']
    optional_str_params = ['--server_password', '--key_file']
    optional_bool_params = ['--websocket', '--cronjob']
    host = None
    user = None
    server_password = None
    key_file = None
    project_name = None
    git_user_password = None
    has_websockets = None
    has_cronjobs = None
    connection = None

    def restart_services(self):
        self.connection.run(f'systemctl restart gunicorn_{self.project_name}')
        print('gunicorn restarted')
        self.connection.run('systemctl restart nginx')
        print('nginx restarted')
        if self.has_websockets:
            self.connection.run('systemctl restart daphne')
            print('daphne restarted')

    def add_arguments(self, parser):
        for arg in self.str_params:
            parser.add_argument(arg, nargs='?', type=str)
        for arg in self.optional_bool_params:
            parser.add_argument(arg, nargs='?', type=bool)
        for arg in self.optional_str_params:
            parser.add_argument(arg, nargs='?', type=str)

    def make_migrations(self, initial=False):
        if initial:
            output = self.connection.run(
                'source ../env/bin/activate && python manage.py makemigrations core && python manage.py makemigrations  && python manage.py migrate')
        else:
            output = self.connection.run(
                'source ../env/bin/activate && python manage.py makemigrations && python manage.py migrate')

        self.show_output(output)

    def get_static_files(self):
        static_response = Responder(pattern="Type 'yes' to continue, or 'no' to cancel:", response='yes\n')
        output = self.connection.run(
            'source ../env/bin/activate python manage.py collectstatic',
            pty=True, watchers=[static_response])
        self.show_output(output)

    def get_new_files(self):
        git_password = Responder(pattern="Password for 'https://LucasLGsing@bitbucket.org':",
                                 response=f'{self.git_user_password}\n')
        output = self.connection.run('git pull', pty=True, watchers=[git_password])
        self.show_output(output)

    def enable_cronjobs(self):
        if self.has_cronjobs:
            self.connection.run(
                'source ../env/bin/activate && python manage.py crontab remove && python manage.py crontab add')

    def update_project(self):
        self.get_new_files()
        self.make_migrations()
        self.get_static_files()
        self.enable_cronjobs()
        self.connection.run(
            f'source env/bin/activate && pip3 install -r {self.project_name}/requirements.txt')

    def show_output(self, output):
        for line in output.stdout.split("\r\n"):
            print(line)

    def get_params(self, kwargs):
        print(kwargs)
        self.project_name = kwargs['project_name']
        self.host = kwargs['host']
        self.user = kwargs['user']
        self.git_user_password = kwargs['git_user_password']
        if kwargs['server_password']:
            self.server_password = kwargs['server_password']
        if kwargs['key_file']:
            self.key_file = kwargs['key_file']

        self.has_websockets = kwargs.get('websockets', None)
        self.has_cronjobs = kwargs.get('cronjobs', None)

    def handle(self, *args, **kwargs):
        self.get_params(kwargs)
        conn_kwargs = {'password': self.server_password} if self.server_password else {'key_filename': self.key_file}
        self.connection = Connection(host=self.host, user=self.user, port=22,
                                     connect_kwargs=conn_kwargs)



from django.core.management import call_command


def update_test_with_key(project_name, host, user, git_user_password, key_file):
    call_command('update_project', project_name, host, user, git_user_password, key_file=key_file)


def update_test_with_password(project_name, host, user, git_user_password, server_password):
    call_command('update_project', project_name, host, user, git_user_password, server_password=server_password)


def build_test_with_key(project_name, host, project_port, user, git_user_password, key_file):
    call_command('build_project', project_name, host, project_port, user, git_user_password, key_file=key_file)


def build_test_with_password(project_name, host, project_port, user, git_user_password, server_password):
    call_command('build_project', project_name, host, project_port, user, git_user_password,
                 server_password=server_password)



import os

from django.conf import settings
from django.core.mail import send_mail
from invoke import Responder, UnexpectedExit

from singular_server_management.management.commands.server_base_command import ServerBaseCommand


class Command(ServerBaseCommand):
    SERVICES_FOLDER = '/etc/systemd/system/'
    NGINX_FOLDER = '/etc/nginx/sites-available/'
    help = 'Update test server'
    str_params = ['project_name', 'host', 'port', 'user', 'git_user_password']
    files_path = f'{os.path.dirname(__file__)}/../../file_templates'

    def send_email(self):
        send_mail(f'Failed to build {self.host} on project {self.project_name}', str(e),
                  settings.DEFAULT_FROM_EMAIL,
                  settings.ADMINS)

    def get_params(self, kwargs):
        super(Command, self).get_params(kwargs)
        self.project_port = kwargs['port']

    def upload_temp_file(self, name, content, location):
        filename = f'{self.files_path}/{name}'
        with open(filename, 'w') as temp:
            try:
                temp.write(content)
            except Exception as e:
                print(e)
        try:
            self.connection.put(filename, location)
        except:
            pass
        try:
            self.connection.run(f'sudo systemctl enable {location}{name}')
        except:
            pass
        try:
            os.remove(filename)
        except:
            pass

    def configure_gunicorn(self):
        try:
            socket_file = open(f'{self.files_path}/gunicorn_default.socket', 'r')
            service_file = open(f'{self.files_path}/gunicorn_default.service', 'r')
            socket_content = socket_file.read()
            service_content = service_file.read()
            socket_content = socket_content.replace('default', self.project_name)
            service_content = service_content.replace('default', self.project_name)
            self.upload_temp_file(f'gunicorn_{self.project_name}.socket', socket_content, self.SERVICES_FOLDER)
            self.upload_temp_file(f'gunicorn_{self.project_name}.service', service_content, self.SERVICES_FOLDER)
        except Exception as e:
            print(e)

    def configure_daphne(self):
        try:
            service_file = open(f'{self.files_path}/daphne.service', 'r')
            service_content = service_file.read()
            service_content = service_content.replace('default', self.project_name)
            self.upload_temp_file(f'daphne.service', service_content, self.SERVICES_FOLDER)
        except Exception as e:
            print(e)

    def configure_nginx(self):
        try:
            nginx_file = open(f'{self.files_path}/nginx_file')
            nginx_content = nginx_file.read()
            nginx_content = nginx_content.replace('default', self.project_name).replace('port',
                                                                                        self.project_port).replace(
                'host', self.host)
            try:
                self.upload_temp_file(self.project_name, nginx_content, self.NGINX_FOLDER)
            except:
                pass
            try:
                self.connection.run(
                    f' sudo ln -s /etc/nginx/sites-available/{self.project_name} /etc/nginx/sites-enabled/{self.project_name}')
            except:
                pass
        except Exception as e:
            print(e)

    def handle(self, *args, **options):
        super(Command, self).handle(*args, **options)
        try:
            self.connection.run(f'cd /var/www/django_apps/{self.project_name}')
        except:
            try:
                self.connection.run(f'mkdir /var/www/django_apps/{self.project_name}')
            except:
                self.connection.run(f'mkdir /var/www/django_apps')
                self.connection.run(f'mkdir /var/www/django_apps/{self.project_name}')
                print('PLEASE MAKE THE INITIAL CONFIGURATION OF THE SERVER')
                print(
                    'RUN sudo apt install git python3-pip python3-dev libpq-dev postgresql postgresql-contrib nginx curl')
                return
        with self.connection.cd(f'/var/www/django_apps/{self.project_name}/'):
            try:
                git_clone_responder = Responder(pattern="Password for 'https://LucasLGsing@bitbucket.org':",
                                                response=f'{self.git_user_password}\n')
                self.connection.run(f'git clone https://LucasLGsing@bitbucket.org/singular-dev/{self.project_name}.git',
                                    pty=True,
                                    watchers=[git_clone_responder])
            except UnexpectedExit as e:
                print(e)
                if 'already exists and is not an empty directory' in e.result.stdout:
                    pass
                else:
                    return
            try:
                self.connection.run(f'cd env/')
            except:
                self.connection.run(f'virtualenv env')
                pass
            with self.connection.cd(f'/var/www/django_apps/{self.project_name}/{self.project_name}'):
                self.connection.run(f'git checkout {getattr(settings, "SERVER_BRANCH", "server")}')
                self.connection.run(
                    f'source ../env/bin/activate && pip3 install -r requirements.txt')
                self.configure_gunicorn()
                if self.has_websockets:
                    self.configure_daphne()
                self.configure_nginx()
                self.connection.run(f'sudo systemctl daemon-reload')
                try:
                    self.make_migrations(True)
                    self.get_static_files()
                except Exception as e:
                    print(e)
                    print(
                        'PLEASE ACCESS THE SERVER TO ADD THE ENVIRONMENT FILE AND ASSOCIATE THE DATABASE')
                    return
        print(
            '!!!!!!!!!!!!!!!!IF REDIS,DAPHNE AND GUNICORN ARE NOT IN YOU REQUIREMENTS, PLEASE INSTALL THEM MANUALLY!!!!!!!!!!!!!!!!!!!!!!')
        print('PROJECT BUILT SUCCESSFULLY!')
        print('AFTER THAT, PLEASE RESTART ALL THE SERVICES')



from django.conf.urls import url
from django.views.decorators.csrf import csrf_exempt

from singular_server_management.webhooks.webhooks import PushRequest

urlpatterns = [
    url('webhooks/push_request', csrf_exempt(PushRequest.as_view()))
]



import json

from django.conf import settings
from django.core.mail import send_mail
from django.core.management import call_command
from django.http import HttpResponse
from django.views.generic.base import View


class PushRequest(View):
    def post(self, *args, **kwargs):
        status = 200
        server_settings = settings.SERVER_SETTINGS
        email_title = f'Server update status at {server_settings["PROJECT_NAME"]}-{server_settings["HOST"]}'
        try:
            body = json.loads(self.request.body.decode('utf-8'))
            branch_name = body['push']['changes'][0]['new']['name']
            if branch_name == settings.SERVER_BRANCH:
                if server_settings['PASSWORD']:
                    call_command('update_server', server_settings['PROJECT_NAME'], server_settings['HOST'],
                             server_settings['USER'],
                             server_settings['GIT_PASSWORD'], f'--server_password {server_settings["PASSWORD"]}')
                else:
                    call_command('update_server', server_settings['PROJECT_NAME'], server_settings['HOST'],
                                 server_settings['USER'],
                                 server_settings['GIT_PASSWORD'], f'--key_file {server_settings["KEY_FILE"]}')
                send_mail(email_title,
                          'Server updated successfully!', settings.DEFAULT_FROM_EMAIL, settings.ADMINS)
        except Exception as e:
            status = 500
            send_mail(email_title, str(e),
                      settings.DEFAULT_FROM_EMAIL, settings.ADMINS)
        return HttpResponse(status=status)
